function [g0,g1,CC,Psi,Pie,H,C,E,ssvalues,ssnames,bad_steady_state] = model_wgcut(param,bnames,obs)
% function [g0,g1,CC,Psi,Pie,H,C,E,ssvalues,ssnames] = model(param,obs)
% For given parameter values, this function puts the model in Chris Sims' 
% Gensys' canonical form and sets up the observation equation.
% The solution will take the form:  x(t) = G * x(t-1) + M * epsilon(t)
%                                   y(t) = H * x(t) + C
%                                   epsilon(t) ~ N(0,E*E')
% INPUT:    
%       param: parameter values
%       obs: vector defining observable variables

% -------------------------------------------------------------------------
% Initializations 
% ------------------------------------------------------------------------- 
nvar = 68;          % number of variables
nexog = 12;         % number of exogenous shocks
nforerrors = 7;     % number of forecast errors

%----------------------------------
% Define variables
%----------------------------------

declare_variable;

%----------------------------------
% Set Parameters
%----------------------------------
                     
pnames = { 	'alph  '   	% 1 
            'bet   '    % 2
            'gamm100  ' % 3
            'kappa  '   % 4
            'omegaw'    % 5    
            'omegap'    % 6
            'etaw'      % 7
            'etap'      % 8
            'phipi'     % 9
            'phiy'      % 10
            'sg'        % 11
            'slg'       % 12
            'pfy'       % 13  
            'tau'       % 14
            'tauc'      % 15
            'thet'      % 16
            'gpsi'      % 17
            's'         % 18 
            'chiw'      % 19
            'chip'      % 20
            'delt'      % 21
            'rhoa'		% 22 
            'rhob'      % 23
            'rhor'      % 24
            'rhoi'      % 25
            'rholab'    % 26
            'rhow'      % 27 
            'rhop'      % 28 
            'rhog'		% 29
            'rhot'      % 30
            'rhotc'     % 31
            'rhoz'      % 32
            'gammg'     % 33
            'gammt'     % 34
            'gammtc'    % 35
            'gammwg'    % 36
            'psit'      % 37
            'psig'      % 38
            'siga'      % 39    
            'sigb'      % 40
            'sigm'      % 41
            'sigi'      % 42
            'siglab'    % 43
            'sigw'      % 44
            'sigp'      % 45
            'sigg'      % 46
            'sigt'      % 47 
            'sigtc'     % 48
            'sigz'      % 49
            'sigwg'     % 50
            'rhowg'     % 51
            'kappawp'   % 52
            'mun'       % 53
            'wpwg'      % 54
            'nu'        % 55
            'kimballp'  % 56
            'kimballw'  % 57
            'alphag'    % 58
            'thetaep'   % 59
            'thetaew'   % 60
            'thetaL'    % 61
                  };      

              
parameters = [0.3                                               % 1. alph: capital share
             0.99                                               % 2. bet: time preference
             param(strmatch('gamm100',bnames,'exact'),:)        % 3. gamm100: steady state growth rate of technology, multiplied by 100
             param(strmatch('kappa',bnames,'exact'),:)          % 4. kappa: curvature over leisure
             param(strmatch('omegaw',bnames,'exact'),:)         % 5. omegaw: prob that union cannot reset optimal wage 
             param(strmatch('omegap',bnames,'exact'),:)         % 6. omegap: prob that intermediat firms cannot reset optimal price  
             0.14                                               % 7. etaw: elasticity of substitution b/w labor talents
             0.14                                               % 8. etap: elasticity of substitution b/w intermediate goods
             param(strmatch('phipi',bnames,'exact'),:)          % 9. phipi: Taylor rule inflation coefficient
             param(strmatch('phiy',bnames,'exact'),:)           % 10.phiy: Taylor rule output coefficient
             0.045                                              % 11. sg: government goods pucrhased to output ratio, S&L G; total sg=.094
             0.077                                              % 12. slg: compensation for public employees to output ratio, S&L G; total slg=.095  
             0.005                                              % 13. pfy pri. fis balance/output %sb, S&L G; totl sb=.56*4 
             .028                                               % 14. tau: income tax rate, S&L G; tota tau=0.186
             0.108                                              % 15. tau^C: consumption tax rate, S&L G; total tauc^c=0.123
             param(strmatch('theta',bnames,'exact'),:)          % 16. theta: habit formation parameter
             param(strmatch('gpsi',bnames,'exact'),:)           % 17. gpsi: K utilization cost parameter; 1: no K utilization
             param(strmatch('s',bnames,'exact'),:)              % 18. s: s" in the investment adjustment cost function
             param(strmatch('chiw',bnames,'exact'),:)           % 19. chiw
             0                                                  % 20. chip: the percent of intermediate firms' price adjustment when they cannot optimize 
             .025                                               % 21. delt: k depreciation
             param(strmatch('rhoa',bnames,'exact'),:)           % 22. rhoa: serial correlation on A 
             param(strmatch('rhob',bnames,'exact'),:)           % 23. rhob: serial correlation on cons. preference (ub)
             param(strmatch('rhor',bnames,'exact'),:)           % 24. rhor: serial correlation on interest rate in Taylor rule
             param(strmatch('rhoi',bnames,'exact'),:)           % 25. rhoi: serial correlation on ui
             param(strmatch('rholab',bnames,'exact'),:)         % 26. rhoulab: serial correlation on ulab
             param(strmatch('rhow',bnames,'exact'),:)           % 27. rhow: serial correlation on uw
             param(strmatch('rhop',bnames,'exact'),:)           % 28. rhop: serial correlation on up
             param(strmatch('rhog',bnames,'exact'),:)           % 29. rhog: serial correlation on g
             param(strmatch('rhot',bnames,'exact'),:)           % 30. rhot: serial correlation on tau
             param(strmatch('rhotc',bnames,'exact'),:)          % 31. rhotc: serial correlation on tau^C
             param(strmatch('rhoz',bnames,'exact'),:)           % 32. rhoz: serial correlation on transfers
             param(strmatch('gammg',bnames,'exact'),:)          % 33. gammg: response of G to debt; 
             param(strmatch('gammt',bnames,'exact'),:)          % 34. gammt: response of income tax revenue to debt; 
             param(strmatch('gammtc',bnames,'exact'),:)         % 35. gammtc: response of consumption tax revenue to debt; 
             param(strmatch('gammwg',bnames,'exact'),:)         % 36. gammwg 
             0                                                  % 37. psit: tau response to output
             0                                                  % 38. psigc: GC response to output, BP value
             param(strmatch('siga',bnames,'exact'),:)           % 39. a sigma 
             param(strmatch('sigb',bnames,'exact'),:)           % 40. b sigma 
             param(strmatch('sigm',bnames,'exact'),:)           % 41. m sigma
             param(strmatch('sigi',bnames,'exact'),:)           % 42. i sigma
             param(strmatch('siglab',bnames,'exact'),:)         % 43  lab sigma
             param(strmatch('sigw',bnames,'exact'),:)           % 44. w sigma
             param(strmatch('sigp',bnames,'exact'),:)           % 45. p sigma
             param(strmatch('sigg',bnames,'exact'),:)           % 46. g sigma
             param(strmatch('sigt',bnames,'exact'),:)           % 47. tau sigma
             param(strmatch('sigtc',bnames,'exact'),:)          % 48. tau^C sigma
             param(strmatch('sigz',bnames,'exact'),:)           % 49. z sigma 
             param(strmatch('sigwg',bnames,'exact'),:)          % 50. wg sigma
             param(strmatch('rhowg',bnames,'exact'),:)          % 51. rhowg, serial correlation on wg 
             param(strmatch('kappawp',bnames,'exact'),:)        % 52. kappawp
             param(strmatch('mun',bnames,'exact'),:)            % 53. mun, elasticity of substitution between the labor used in private and public sectors
             0.69                                               % 54. wpwg, wp/wg-total comp; wp/wg-w&s=0.74
             0                                                  % 55. nu, productivity of govt. production
             10                                                 % 56. kimballp, curvature on goods Kimball aggregator
             10                                                 % 57. kimballw, curvature on labor Kimball aggregator
             0                                                  % 58. alphag, yg in utility, substitutability (>0) or complementarity (< 0) w/ private consumption
             param(strmatch('thetaep',bnames,'exact'),:)        % 59. thetaep, MA term on price markup process
             param(strmatch('thetaew',bnames,'exact'),:)        % 60. thetaew, MA term on wage markup process
             param(strmatch('thetaL',bnames,'exact'),:)         % 61. thetaL, governs strength of short-term wealth effect on labor; 1 = baseline model 
             ];
             

for j = 1:length(parameters)
    eval([char(pnames(j,:)) '=parameters(j);']);
end


%----------------------------------
% Solve for the steady state
%----------------------------------
gamm = gamm100/100;
expg = exp(gamm);

rk = (expg - bet*(1 - delt))/(bet*(1 - tau));
R = expg/bet;
rreal = R;
psi1 = rk*(1-tau);
ygy = slg;
ypy = 1-ygy;
mc = 1/(1 + etap);
Omegyp = etap;
wp = (((1-alph)^(1-alph))*(alph^(alph))*(rk^(-alph))*((ygy/(ypy + Omegyp*ypy))^(nu/(1-nu)))*mc)^(1/(1-alph));
wg = wp/wpwg;
lgy = slg/(wg);
klp = (wp*alph/(rk*(1-alph)));
yplp = ((1/(1+Omegyp))*(klp^(alph*(1-nu)))*((ygy/ypy)^nu))^(1/(1-nu));
lpyp = yplp^(-1);
lgyp = lgy/ypy;
lplg = lpyp/lgyp;
ivyp = (1 - (1-delt)*exp(-gamm))*expg*klp*lpyp;
cyp = 1-ivyp-sg/ypy;
cstaryp = cyp + alphag*ygy/ypy;
varphi = (((lplg)/((wp/wg)^mun))+1)^(-1);
w = ((1-varphi)*(wp^(1+mun))+varphi*(wg^(1+mun)))^(1/(1+mun));
lyp = lpyp/((1-varphi)*((wp/w)^(mun)));
yp = ((w*(1-tau)*(((1-thet*exp(-gamm))*cstaryp)^(-thetaL))*(exp(-gamm)^(1-thetaL)))/((1+tauc)*(1+etaw)*(lyp^kappa)))^(1/(thetaL+kappa));
y = (1/ypy)*yp;

sb = pfy/(R/expg-1);
str = tau*w*lyp*ypy + tau*rk*klp*lpyp*ypy + tauc*cyp*ypy - slg - sg - sb*(R/expg-1);

c = cyp*yp;
cstar = cstaryp*yp;
inv_ss = ivyp*yp;
yg = ygy*y;
lp = lpyp*yp;
k = klp*lp;
l = lyp*yp;
Omeg = Omegyp*yp;
lg = slg/(wg/y);
z = str*y;
b = sb*y;
lambr = 1/((1-thet*exp(-gamm))*cstar*(1+tauc));
t = (w*l+rk*k)*tau;
tc = tauc*c;
GT = (sg+slg)*y;
gd = sg*y;
fs = t+tc-GT-z;
fd = -fs;
if wp < 0 || c < 0 || inv_ss < 0 || lp < 0 || l < 0 || yp < 0 || w < 0 || varphi < 0 ...
        || (varphi^(-1/mun)) == Inf || ((1-varphi)^(-1/mun)) == Inf
    bad_steady_state = 1;
    g0 = []; g1 = []; CC = []; Psi = []; Pie = []; H = []; C = []; E = []; ssvalues = []; ssnames = [];
else
    bad_steady_state = 0;
    
ssvalues = [rk R psi1 wp wg ygy ypy lpyp klp cyp str y l Omeg w lp lg yg yp gd GT k inv_ss z b c lambr t rreal mun varphi sb fs fd tc cstar];
ssnames = {'rk';'R';'psi1';'wp';'wg';'ygy';'ypy';'lpyp';'klp';'cyp';'str';'y';'l';'Omeg';'w';...
           'lp';'lg';'yg';'yp';'gd';'GT';'k';'inv_ss';'z';'b';'c';'lambr';'t';'rreal'; 'mun';'varphi';'sb';'fs';'fd';'tc';'cstar'};
         

        
% -------------------------------------------------------------------------
% Standard deviation matrix 
% -------------------------------------------------------------------------
E = diag([siga/100 sigb/100 sigm/100 sigi/100 siglab/100 sigw/100 sigp/100 sigg/100 sigt/100 sigtc/100 sigz/100 sigwg/100]);


%% ------------------------------------------------------------------------
% System Matrices 
% -------------------------------------------------------------------------
g0 = zeros(nvar,nvar);
g1 = g0;
CC = zeros(nvar,1);
Psi = zeros(nvar,nvar);
Pie = zeros(nvar,nforerrors);


%--------------------------------------------
%	Lagrange Multiplier
%--------------------------------------------
g0(Nlambda,Nlambda) = 1;
g0(Nlambda,Nua) = thet/(expg-thet);
g0(Nlambda,Ncstar) = expg/(expg-thet);
g1(Nlambda,Ncstar) = thet/(expg-thet);
g0(Nlambda,Nub) = -1;
g0(Nlambda,Ntauc) = tauc/(1+tauc);

%--------------------------------------------
%	Z function in Utility
%--------------------------------------------
g0(NzL,NzL) = 1;
g1(NzL,NzL) = 1-thetaL;
g0(NzL,Nua) = 1-thetaL - thetaL*thet/(expg-thet);
g0(NzL,Ncstar) = -thetaL*expg/(expg-thet);
g1(NzL,Ncstar) = -thetaL*thet/(expg-thet);

%--------------------------------------------
%	Consumption in Utility
%--------------------------------------------
g0(Ncstar,Ncstar) = 1;
g0(Ncstar,Nc) = -c/(cstar);
g0(Ncstar,Nyg) = -alphag*yg/(cstar);

%--------------------------------------------
%	Euler Equation for Bonds
%--------------------------------------------
g0(Nc,Nlambda) = 1;
g0(Nc,NR) = -1;
g0(Nc,Nxpi) = 1;
g0(Nc,Nxlambda) = -1;
g0(Nc,Nua) = rhoa;

%--------------------------------------------
%	Investment FOC
%--------------------------------------------
g0(Ni,Nq) = -1/((1+bet)*s*(exp(2*gamm)));
g0(Ni,Ni) = 1;
g0(Ni,Nxi) = -bet/(1+bet);
g0(Ni,Nua)=(1-bet*rhoa)/(1+bet);
g1(Ni,Ni) = 1/(1+bet);
g0(Ni,Nui) = -1;        % Normalization: ui = [1/((1+bet)*s*(exp(2*gamm)))]*ui_original

%--------------------------------------------
%	Capital FOC
%--------------------------------------------
g0(Nq,Nq) = 1;
g0(Nq,NR) = 1;
g0(Nq,Nxpi) = -1;
g0(Nq,Nxq) = -bet*exp(-gamm)*(1-delt);
g0(Nq,Nxrk) = -bet*exp(-gamm)*rk*(1-tau);
g0(Nq,Nxtau) = tau*exp(-gamm)*bet*rk;

%--------------------------------------------
%	Law of Motion for Capital
%--------------------------------------------
g0(Nkbar,Nkbar) = 1;
g0(Nkbar,Nui) = -(1 - (1-delt)*exp(-gamm))*((1+bet)*s*(exp(2*gamm))); 
g0(Nkbar,Ni) = -(1 - (1-delt)*exp(-gamm));
g0(Nkbar,Nua) = (1-delt)*exp(-gamm);
g1(Nkbar,Nkbar) = (1-delt)*exp(-gamm);

%--------------------------------------------
%	Phillips Equation
%--------------------------------------------
lamprice = ((1-bet*omegap)*(1-omegap))/((1+bet*chip)*omegap*(1 + etap*kimballp));
g0(Npi,Npi) = 1;
g0(Npi,Nxpi) = -bet/(1+bet*chip);
g1(Npi,Npi) = chip/(1+bet*chip);
g0(Npi,Nmc) = -lamprice;
g0(Npi,Nup) = -1;    % Normalization: up = lamprice*up_original   

%--------------------------------------------
%	Wage Equation
%--------------------------------------------
lamwage = ((1-omegaw*bet)*(1-omegaw))/(omegaw*(1 + etaw*kimballw)*(1+bet));  
g0(Nw,Nw) = 1;
g0(Nw,Nxw) = -(bet/(1+bet));
g0(Nw,Npi) = ((1 + bet*chiw)/(1 + bet));
g0(Nw,Nxpi) = -(bet/(1+bet));
g0(Nw,Nunemp) = lamwage*kappa;
g0(Nw,Nuw) = -1;     % Normalization: uw = (lamwage/(1+lamwage))*uw_original
g0(Nw,Nua) = ((1 + bet*chiw - rhoa*bet)/(1+bet));
g1(Nw,Nw) = (1/(1+bet));
g1(Nw,Npi) = (chiw/(1 + bet));
g1(Nw,Nua) = (chiw/(1+bet));

%--------------------------------------------
%	Unemployment-Labor Link Equation
%--------------------------------------------
g0(Nunemp,Nunemp) = kappa;
g0(Nunemp,Nw) = -1;
g0(Nunemp,NzL) = 1;
g0(Nunemp,Nl) = kappa;
g0(Nunemp,Ntauc) = tauc/(1+tauc);
g0(Nunemp,Ntau) = (tau/(1-tau));
g0(Nunemp,Nulab) = 1;

%--------------------------------------------
%	Production Factors
%--------------------------------------------
g0(Nrk,Nrk) = 1;
g0(Nrk,Nwp) = -1;
g0(Nrk,Nk) = 1;
g0(Nrk,Nlp) = -1;

%--------------------------------------------
%	Private Production Function
%--------------------------------------------
g0(Nlp,Nyp) = 1;
g0(Nlp,Nk) = -((yp+Omeg)/yp)*alph*(1-nu);
g0(Nlp,Nlp) = -((yp+Omeg)/yp)*(1-alph)*(1-nu);
g0(Nlp,Nyg) = -((yp+Omeg)/yp)*nu;

%--------------------------------------------
%	Goods market clearing
%--------------------------------------------
g0(Nyp,Nc) = c;
g0(Nyp,Ni) = inv_ss;
g0(Nyp,Nyp) = -yp;
g0(Nyp,Ngd) = sg*y;
g0(Nyp,Nv) = psi1*k;

%--------------------------------------------
%	Government Budget Constraint
%--------------------------------------------
g0(Nb,Nb) = 1;
g0(Nb,Ntauc) = tauc*(c/y)/sb;
g0(Nb,Nc) = tauc*(c/y)/sb;
g0(Nb,Ntau) = (tau*rk*k/y + tau*w*l/y)/sb;
g0(Nb,Nw) = (tau*w*l/y)/sb;
g0(Nb,Nl) = (tau*w*l/y)/sb;
g0(Nb,Nrk) = (tau*rk*k/y)/sb;
g0(Nb,Nk) = (tau*rk*k/y)/sb;
g0(Nb,Npi) = (R/expg);
g0(Nb,Nz) = -1;  % normalized so z = str/sb*z_original
g0(Nb,Ngd) = -(gd/y)/sb;
g0(Nb,Nwg) = -(wg*lg/y)/sb;
g0(Nb,Nlg) = -(wg*lg/y)/sb;
g1(Nb,NR) = (R/expg);
g1(Nb,Nb) = (R/expg);

g0(Nb,Nua) = (R/expg);

%--------------------------------------------
%	v equation
%-------------------------------------------- 
g0(Nv,Nrk) = ((1-gpsi)/gpsi);
g0(Nv,Nv) = -1;
g0(Nv,Ntau) = -(((1-gpsi)/gpsi)*tau/(1-tau));

%--------------------------------------------
%	k, effective unit of capital
%--------------------------------------------
g0(Nk,Nk) = 1;
g0(Nk,Nv) = -1;
g0(Nk,Nua) = 1;
g1(Nk,Nkbar) = 1;

%--------------------------------------------
%	Marginal Cost
%--------------------------------------------
g0(Nmc,Nmc) = 1;
g0(Nmc,Nrk) = -alph;
g0(Nmc,Nwp) = alph-1;
g0(Nmc,Nyg) = nu/(1-nu);
g0(Nmc,Nyp) = -(nu/(1-nu))*(yp/(yp + Omeg));

%--------------------------------------------
%	Taylor Rule
%--------------------------------------------
g0(NR,NR) = 1;
g1(NR,NR) = rhor;
g0(NR,Npi) = -(1-rhor)*phipi;
g0(NR,Ny) = -(1-rhor)*phiy;
g0(NR,Num) = -1;

%--------------------------------------------
%	sb Defined
%--------------------------------------------
g0(Nsb,Nsb) = 1;
g0(Nsb,Ny) = 1;
g0(Nsb,Nb) = -1;

%--------------------------------------------
%    gd Rule
%--------------------------------------------
g0(Ngd,Ngd) = 1;
g1(Ngd,Nsb) = -gammg;
%g1(Ngd,Nsfd) = -gammg;
g1(Ngd,Ngd) = rhog;
g0(Ngd,Ny) = psig; 
g0(Ngd,Nug) = -1;

%--------------------------------------------
%     Income Tax Rate Rule
%--------------------------------------------
g0(Ntau,Ntau) = 1;
g1(Ntau,Nsb) = gammt;
g1(Ntau,Ntau) = rhot;
g0(Ntau,Ny) = -psit; 
g0(Ntau,Nut) = -1;

%--------------------------------------------
%     Consumption Tax Rate Rule
%--------------------------------------------
g0(Ntauc,Ntauc) = 1;
g1(Ntauc,Nsb) = gammtc;
g1(Ntauc,Ntauc) = rhotc;
g0(Ntauc,Nutc) = -1;

%--------------------------------------------
%    Z Rule
%--------------------------------------------
g0(Nz,Nz) = 1;
g1(Nz,Nz) = rhoz;
g0(Nz,Nuz) = -1;


%----------------------------------------------
%   Wg rule 
%----------------------------------------------
g0(Nwg,Nwg) = 1;
g1(Nwg,Nwg) = rhowg;
g1(Nwg,Nwp) = kappawp;
g1(Nwg,Nsb) = -gammwg;
g0(Nwg,Nuwg) = -1;

%--------------------------------------------
%	Fisher Equation: real interest rate checking
%--------------------------------------------
g0(Nr,Nr) = 1;
g0(Nr,NR) = -1;
g0(Nr,Nxpi) = 1;

%--------------------------------------------
%	Income Tax Revenue
%--------------------------------------------
g0(Nt,Nt) = t;
g0(Nt,Ntau) = -(w*l+rk*k)*tau;
g0(Nt,Nw) = -w*l*tau;
g0(Nt,Nl) = -w*l*tau;
g0(Nt,Nrk) = -rk*k*tau;
g0(Nt,Nk) = -rk*k*tau;

%--------------------------------------------
%	Consumption Tax Revenue
%--------------------------------------------
g0(Ntc,Ntc) = 1;
g0(Ntc,Ntauc) = -1;
g0(Ntc,Nc) = -1;

%----------------------------------------------------
%   yg = G -  =wg*lg
%----------------------------------------------------
g0(Nyg,Nyg) = 1;
g0(Nyg,Nwg) = -1;
g0(Nyg,Nlg) = -1;

%----------------------------------------------------
%   total output 
%-----------------------------------------------------
g0(Ny,Ny) = y;
g0(Ny,Nyg) = -yg;
g0(Ny,Nyp) = -yp;

%--------------------------------------------
%	total government spending GT=g+wg*lg
%--------------------------------------------
g0(NGT,NGT) = GT;
g0(NGT,Ngd) = -gd;
g0(NGT,Nlg) = -wg*lg;
g0(NGT,Nwg) = -wg*lg;

%----------------------------------------------
%    aggregate L 
%----------------------------------------------
g0(Nl,Nl) = l^((1+mun)/mun);
g0(Nl,Nlp) = -((1-varphi)^(-1/mun))*(lp^((1+mun)/mun));
g0(Nl,Nlg) = -(varphi^(-1/mun))*(lg^((1+mun)/mun));  

%--------------------------------------------
%	lg 
%--------------------------------------------
g0(Nlg,Nlg) = 1;
g0(Nlg,Nwg) = -mun;
g0(Nlg,Nw) = mun;
g0(Nlg,Nl) = -1;

%---------------------------------------------------
%   aggregate wage (w) 
%---------------------------------------------------
g0(Nwp,Nw) = w^(1+mun);
g0(Nwp,Nwp) = -(1-varphi)*(wp^(1+mun));
g0(Nwp,Nwg) = -varphi*(wg^(1+mun));

%--------------------------------------------
%primary fiscal surplus
%--------------------------------------------
g0(Nfs,Nfs) = fs;
g0(Nfs,Nt) = -t;
g0(Nfs,Ntc) = -tauc*c;
g0(Nfs,Nz) = z;
g0(Nfs,NGT) = GT;

%--------------------------------------------
%fiscal surplus
%--------------------------------------------
g0(Nfd,Nfd) = 1;
g0(Nfd,Nfs) = 1;

%--------------------------------------------
%	Define Xpi
%--------------------------------------------
g0(Nxpi,Npi)=1; 
g1(Nxpi,Nxpi)=1; 
Pie(Nxpi,1)=1;

%--------------------------------------------
%	Define Xw
%--------------------------------------------
g0(Nxw,Nw)=1; 
g1(Nxw,Nxw)=1; 
Pie(Nxw,2)=1;

%--------------------------------------------
%	Define Xq
%--------------------------------------------
g0(Nxq,Nq)=1; 
g1(Nxq,Nxq)=1; 
Pie(Nxq,3)=1;

%--------------------------------------------
%	Define Xrk
%--------------------------------------------
g0(Nxrk,Nrk)=1; 
g1(Nxrk,Nxrk)=1; 
Pie(Nxrk,4)=1;

%--------------------------------------------
%	Define Xi
%--------------------------------------------
g0(Nxi,Ni)=1; 
g1(Nxi,Nxi)=1; 
Pie(Nxi,5)=1;

%--------------------------------------------
%	Define Xlambda
%--------------------------------------------
g0(Nxlambda,Nlambda)=1; 
g1(Nxlambda,Nxlambda)=1; 
Pie(Nxlambda,6)=1;

%--------------------------------------------
%	Define Xtau
%--------------------------------------------
g0(Nxtau,Ntau)=1; 
g1(Nxtau,Nxtau)=1; 
Pie(Nxtau,7)=1;

%--------------------------------------------
%	C Obs
%--------------------------------------------
g0(Ncobs,Ncobs) = 1;
g0(Ncobs,Nc) = -100;
g1(Ncobs,Nc) = -100;
g0(Ncobs,Nua) = -100;

%--------------------------------------------
%	I Obs
%--------------------------------------------
g0(Niobs,Niobs) = 1;
g0(Niobs,Ni) = -100;
g1(Niobs,Ni) = -100;
g0(Niobs,Nua) = -100;

%--------------------------------------------
%	T Obs
%--------------------------------------------
g0(NTobs,NTobs) = 1;
g0(NTobs,Nt) = -100;
g1(NTobs,Nt) = -100;
g0(NTobs,Nua) = -100;

%--------------------------------------------
%	Tc Obs
%--------------------------------------------
g0(NTcobs,NTcobs) = 1;
g0(NTcobs,Ntc) = -100;
g1(NTcobs,Ntc) = -100;
g0(NTcobs,Nua) = -100;

%--------------------------------------------
%	gd Obs
%--------------------------------------------
g0(Ngdobs,Ngdobs) = 1;
g0(Ngdobs,Ngd) = -100;
g1(Ngdobs,Ngd) = -100;
g0(Ngdobs,Nua) = -100;

%--------------------------------------------
%	wp Obs (real private wage growth)
%--------------------------------------------
g0(Nwpobs,Nwpobs) = 1;
g0(Nwpobs,Nwp) = -100;
g1(Nwpobs,Nwp) = -100;
g0(Nwpobs,Nua) = -100;

%--------------------------------------------
%	wg Obs (real public wage growth)
%--------------------------------------------
g0(Nwgobs,Nwgobs) = 1;
g0(Nwgobs,Nwg) = -100;
g1(Nwgobs,Nwg) = -100;
g0(Nwgobs,Nua) = -100;

%--------------------------------------------
%	b Obs
%--------------------------------------------
g0(Nbobs,Nbobs) = 1;
g0(Nbobs,Nb) = -100;
g1(Nbobs,Nb) = -100;
g0(Nbobs,Nua) = -100;

%--------------------------------------------
%	R Obs
%--------------------------------------------
g0(NRobs,NRobs) = 1;
g0(NRobs,NR) = -100;

%--------------------------------------------
%	Pi Obs
%--------------------------------------------
g0(NPiobs,NPiobs) = 1;
g0(NPiobs,Npi) = -100;

%--------------------------------------------
%	Lp Obs
%--------------------------------------------
g0(NLpobs,NLpobs) = 1;
g0(NLpobs,Nlp) = -100;

%--------------------------------------------
%	Unemp Obs
%--------------------------------------------
g0(Nunempobs,Nunempobs) = 1;
g0(Nunempobs,Nunemp) = -100;

%--------------------------------------------
%	Growth Rate of Technology shock
%--------------------------------------------
g0(Nua,Nua) = 1; 
g1(Nua,Nua) = rhoa; 
Psi(Nua,Nua) = 1;

%--------------------------------------------
%	Wage markup process
%--------------------------------------------
g0(Nuw,Nuw) = 1; 
g1(Nuw,Nuw) = rhow; 
g0(Nuw,New) = -1;
g1(Nuw,New) = -thetaew;

%--------------------------------------------
%	Price markup process
%--------------------------------------------
g0(Nup,Nup) = 1; 
g1(Nup,Nup) = rhop;
g0(Nup,Nep) = -1;
g1(Nup,Nep) = -thetaep;

%--------------------------------------------
%	Preference Shock
%--------------------------------------------
ubnorm = ((1-rhob)*(expg-thet)/(expg+thet))*(cstar/c);
g0(Nub,Nub) = 1; 
g1(Nub,Nub) = rhob; 
Psi(Nub,Nub) = 1/ubnorm;
% Normalization: ub = [((1-rhob)*(expg-thet)/(expg+thet))*(cstar/c)]*ub_original;

%--------------------------------------------
%	Monetary Policy Shock
%--------------------------------------------
g0(Num,Num) = 1; 
Psi(Num,Num) = 1;

%--------------------------------------------
%	Investment Shock
%--------------------------------------------
g0(Nui,Nui) = 1; 
g1(Nui,Nui) = rhoi; 
Psi(Nui,Nui) = 1;

%--------------------------------------------
%	Labor preference Shock
%--------------------------------------------
g0(Nulab,Nulab) = 1; 
g1(Nulab,Nulab) = rholab; 
Psi(Nulab,Nulab) = 1;

%--------------------------------------------
%	wage markup Shock
%--------------------------------------------
g0(New,New) = 1;
Psi(New,New) = 1;

%--------------------------------------------
%	price markup Shock
%--------------------------------------------
g0(Nep,Nep) = 1;
Psi(Nep,Nep) = 1;

%--------------------------------------------
%	Income Tax Shock
%--------------------------------------------
g0(Nut,Nut)=1; 
Psi(Nut,Nut) = 1;

%--------------------------------------------
%	Consumption Tax Shock
%--------------------------------------------
g0(Nutc,Nutc)=1; 
Psi(Nutc,Nutc) = 1;

%--------------------------------------------
%	gd Spending Shock
%--------------------------------------------
g0(Nug,Nug) = 1;
Psi(Nug,Nug) = -1/sg;

%--------------------------------------------
%	Z Shock
%--------------------------------------------
g0(Nuz,Nuz)= 1; 
Psi(Nuz,Nuz) = 1;

%--------------------------------------------
%	Public Wage Shock
%--------------------------------------------
g0(Nuwg,Nuwg)= 1; 
Psi(Nuwg,Nuwg) = -1/0.18;


%% End Model Equations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%----------------------------------
% Observation Equation
%----------------------------------
nobs = size(obs,1);

% Relating observables to model variables
H = zeros(nobs,nvar);
for gh = 1:nobs
    H(gh,obs(gh,1)) = 1;
end

% Constants in the observation equation
C = zeros(nobs,1);
C(1) = gamm100;
C(2) = gamm100;
C(3) = gamm100;
C(4) = gamm100;
C(5) = gamm100;
C(6) = gamm100;
C(7) = gamm100;
C(8) = gamm100;
C(9) = 0;
C(10) = 0;
C(11) = 0;
end
